/**
 * Localization strings for the UI Multiselect widget
 *
 * @locale de, de-DE
 */

$.extend($.ui.multiselect.locale, {
	addAll:'Alle hinzufŸgen',
	removeAll:'Alle entfernen',
	itemsCount:'#{count} Elemente gewŠhlt',
	itemsTotal:'#{count} Elemente gesamt',
	busy:'Bitte warten...',
	errorDataFormat:"Option kann nicht hinzugefŸgt werden. Unbekanntes Datenformat.",
	errorInsertNode:"Es gab ein Problem beim hinzufŸgen des Elements:\n\n\t[#{key}] => #{value}\n\nDer Vorgang wurde abgebrochen.",
	errorReadonly:"Die Option #{option} ist schreibgeschŸtzt",
	errorRequest:"Entschuldigung! Es scheint ein Problem mit dem entfernten Aufruf zu geben. (Typ: #{status})"
});
