/* Multiselect
----------------------------------*/

.ui-multiselect { border: 1px solid; font-size: 0.8em; }
.ui-multiselect ul { -moz-user-select: none; }
.ui-multiselect div.list-container { padding: 0; margin: 0; border: 0; float:left;  }
.ui-multiselect ul.list { position: relative; padding: 0; overflow: auto; overflow-x: hidden; margin: 0; list-style: none; width: 100%; }
.ui-multiselect li { margin: 0; padding: 0; cursor: default; line-height: 20px; height: 20px; font-size: 11px; list-style: none; padding-left: 10px; white-space: nowrap; overflow: hidden; }
.ui-multiselect li a { color: #999; text-decoration: none; padding: 0; display: block; float: left; cursor: pointer;}
.ui-multiselect ul.ui-state-highlight { border: none; }
.ui-multiselect li.ui-draggable-dragging { border: 1px solid; padding-left: 10px; z-index: 99999; }

.ui-multiselect div.selected { }
.ui-multiselect ul.selected { }
.ui-multiselect ul.selected li { }

.ui-multiselect div.available { border-left: 1px solid; }
.ui-multiselect ul.available { }
.ui-multiselect ul.available li { }
 
.ui-multiselect .ui-state-default { border: none; margin-bottom: 1px; position: relative; }
.ui-multiselect .ui-state-hover { border: none; }
.ui-multiselect .ui-widget-header {border: none; font-size: 11px; margin-bottom: 1px;}
 
.ui-multiselect .add-all { float: right; padding: 7px; white-space: nowrap; }
.ui-multiselect .remove-all { float: right; padding: 7px; white-space: nowrap; }
.ui-multiselect .count { cursor: default; float: left; padding: 7px; }
.ui-multiselect .search { float: left; padding: 1px; opacity: 0.7; margin: 4px; height: 14px; }
.ui-multiselect .busy { left: 0px; float: left; padding: 7px 7px 7px 25px; z-index: 1; background: url(images/loading.gif) 4px 50% no-repeat; }
.ui-multiselect .overlay { background: #aaaaaa; opacity: .30;filter:Alpha(Opacity=30); }

.ui-multiselect li a.action { position: absolute; border: none; right: 0px; top: 1px; padding-right: 2px; }