/**
 * Localization strings for the UI Multiselect widget
 *
 * @locale zh-CN
 */

$.extend($.ui.multiselect.locale, {
	addAll:'添加全部',
	removeAll:'删除全部',
	itemsCount:'选择 #{count} 项',
	itemsTotal:'共 #{count} 项',
	busy:'请等待……',
	errorDataFormat:"无法添加选项，未知的数据格式",
	errorInsertNode:"在加入下列条目时发生问题：\n\n\t[#{key}] => #{value}\n\n操作失败。",
	errorReadonly:"选项 #{option} 是只读的",
	errorRequest:"抱歉！远程调用似乎有问题。（类型： #{status} ）"
});
